Tabs {
		dock: top;
}
Screen {
	layers: base overlay notes notifications;
	overflow: hidden;
}
QuitScreen {
		align: center middle;
}
TaskComplete {
		align: center top;
}
#dialog {
		grid-size: 2;
		grid-gutter: 1 2;
		grid-rows: 1fr 3;
		padding: 0 1;
		width: 60;
		height: 20;
		border: thick $background 80%;
		background: $surface;
}

#question {
		column-span: 2;
		height: 1fr;
		width: 1fr;
		content-align: center middle;
}

Button {
		width: 100%;
}
TerkaModalScreen {
		align: center top;
}
#edit-dialog {
		grid-size: 1;
		grid-gutter: 1 2;
		grid-rows: 1fr 3;
		padding: 0 1;
		width: 60;
		height: 40;
		border: thick $background 80%;
		background: $surface;
}
.text-input {
	column-span: 2;

}
.description-input {
	column-span: 2;
	height: 20;
}
.small-select {
	column-span: 2;
}
#small-dialog {
		layout: grid;
		grid-size: 2;
		grid-rows: 1fr;
		grid-columns: 0.5fr;
		grid-gutter: 1;
		padding: 0 1;
		width: 40%;
		height: 30%;
		border: thick $background 80%;
		background: $surface;
}
#new-dialog {
		layout: grid;
		grid-size: 2;
		grid-rows: 1fr;
		grid-columns: 0.5fr;
		grid-gutter: 1;
		padding: 0 1;
		width: 90%;
		height: 80%;
		border: thick $background 80%;
		background: $surface;
}
#new-composite-dialog {
		layout: grid;
		grid-size: 2;
		grid-rows: 1fr;
		grid-columns: 0.5fr;
		grid-gutter: 1;
		padding: 0 1;
		width: 60%;
		height: 40%;
		border: thick $background 80%;
		background: $surface;
}
.plotext {
		width: 100%;
		height: 20;
}
.header {
		margin:1 1;
		width: 100%;
		height: 5%;
		background: $panel;
		border: tall $primary;
		content-align: center middle;
}
.new_entity {
	align: left top;
	width: 15%;
}
Sidebar {
	width: 80%;
	background: $panel;
	transition: offset 500ms in_out_cubic;
	layer: overlay;
}
Sidebar.-hidden {
	offset-x: -100%;
}
Sidebar:focus-within {
	offset: 0 0 !important;
}
.completed_sprint {
	color: white;
	background: green;
}
.active_sprint {
	color: white;
	background: blue;
}
.planned_sprint {
	color: white;
	background: black;
}
